@@ -8,15 +8,11 @@ import collections
 import json
 import os
 import shutil
-import warnings
 from pathlib import Path
-from typing import Any, List, MutableMapping, Optional, Union
+from typing import Any, List, Literal, MutableMapping, Optional, Union
 
 import numpy as np
 from huggingface_hub import HfApi, InferenceApi, snapshot_download
-from requests import HTTPError
-
-from ..utils.fixes import Literal
 
 SUPPORTED_TASKS = [
     "tabular-classification",
@@ -153,6 +149,11 @@ def _create_config(
         "text-regression",
     ],
     data,
+    model_format: Literal[  # type: ignore
+        "skops",
+        "pickle",
+        "auto",
+    ] = "auto",
 ) -> None:
     """Write the configuration into a ``config.json`` file.
 
@@ -187,6 +188,13 @@ def _create_config(
 
         The first 3 input values are used as example inputs.
 
+    model_format: str
+        The format used to persist the model. Can be ``"auto"``, ``"skops"``
+        or ``"pickle"``. Defaults to ``"auto"``, which would mean:
+
+        - ``"pickle"`` if the extension is one of ``{".pickle", ".pkl", ".joblib"}``
+        - ``"skops"`` if the extension is ``".skops"``
+
     Returns
     -------
     None
@@ -197,10 +205,22 @@ def _create_config(
     def recursively_default_dict() -> MutableMapping:
         return collections.defaultdict(recursively_default_dict)
 
+    if model_format == "auto":
+        extension = Path(model_path).suffix
+        if extension in [".pkl", ".pickle", ".joblib"]:
+            model_format = "pickle"
+        elif extension == ".skops":
+            model_format = "skops"
+    if model_format not in ["skops", "pickle"]:
+        raise ValueError(
+            "Cannot determine the input file format. Please indicate the format using"
+            " the `model_format` argument."
+        )
     config = recursively_default_dict()
     config["sklearn"]["model"]["file"] = str(model_path)
     config["sklearn"]["environment"] = requirements
     config["sklearn"]["task"] = task
+    config["sklearn"]["model_format"] = model_format
 
     if "tabular" in task:
         config["sklearn"]["example_input"] = _get_example_input(data)
@@ -211,8 +231,7 @@ def _create_config(
         else:
             raise ValueError("The data needs to be a list of strings.")
 
-    with open(Path(dst) / "config.json", mode="w") as f:
-        json.dump(config, f, sort_keys=True, indent=4)
+    dump_json(Path(dst) / "config.json", config)
 
 
 def _check_model_file(path: str | Path) -> Path:
@@ -237,7 +256,7 @@ def _check_model_file(path: str | Path) -> Path:
         raise OSError(f"Model file '{path}' does not exist.")
 
     if os.path.getsize(path) == 0:
-        warnings.warn(f"Model file '{path}' is empty.")
+        raise RuntimeError(f"Model file '{path}' is empty.")
 
     return Path(path)
 
@@ -254,6 +273,11 @@ def init(
         "text-regression",
     ],
     data,
+    model_format: Literal[  # type: ignore
+        "skops",
+        "pickle",
+        "auto",
+    ] = "auto",
 ) -> None:
     """Initialize a scikit-learn based Hugging Face repo.
 
@@ -294,6 +318,10 @@ def init(
         :class:`numpy.ndarray`. If ``task`` is ``"text-classification"`` or
         ``"text-regression"``, the data needs to be a ``list`` of strings.
 
+    model_format: str
+        The format the model was persisted in. Can be ``"auto"``, ``"skops"``
+        or ``"pickle"``. Defaults to ``"auto"`` that relies on file extension.
+
     Returns
     -------
     None
@@ -321,12 +349,73 @@ def init(
             dst=dst,
             task=task,
             data=data,
+            model_format=model_format,
         )
     except Exception:
         shutil.rmtree(dst)
         raise
 
 
+def add_files(*files: str | Path, dst: str | Path, exist_ok: bool = False) -> None:
+    """Add files to initialized repo.
+
+    After having called :func:`.hub_utils.init`, use this function to add
+    arbitrary files to be uploaded in addition to the model and model card.
+
+    In particular, it can be useful to upload the script itself that produces
+    those artifacts by calling ``hub_utils.add_files([__file__], dst=...)``.
+
+    Parameters
+    ----------
+    *files : str or Path
+        The files to be added.
+
+    dst : str or Path
+        Path to the initialized repo, same as used during
+        :func:`.hub_utils.init`.
+
+    exist_ok : bool (default=False)
+        Whether it's okay or not to add a file that already exists. If
+        ``True``, override the files, otherwise raise a ``FileExistsError``.
+
+    Raises
+    ------
+    FileNotFoundError
+        When the target folder or the files to be added are not found.
+
+    FileExistsError
+        When a file is added that already exists at the target location and
+        ``exist_ok=False``.
+
+    """
+    dst = Path(dst)
+    # check dst exists
+    if not dst.exists():
+        msg = f"Could not find '{dst}', did you run 'skops.hub_utils.init' first?"
+        raise FileNotFoundError(msg)
+
+    src_files = [Path(file) for file in files]
+    # check that source files exist
+    for file in src_files:
+        if not file.exists():
+            msg = f"File '{file}' could not be found."
+            raise FileNotFoundError(msg)
+
+    dst_files = [dst / Path(file).name for file in files]
+    for src_file, dst_file in zip(src_files, dst_files):
+        # check if target file already exists
+        if dst_file.exists() and not exist_ok:
+            msg = f"File '{src_file.name}' already found at '{dst}'."
+            raise FileExistsError(msg)
+
+        shutil.copy2(src_file, dst_file)
+
+
+def dump_json(path, content):
+    with open(Path(path), mode="w") as f:
+        json.dump(content, f, sort_keys=True, indent=4)
+
+
 def update_env(
     *, path: Union[str, Path], requirements: Union[List[str], None] = None
 ) -> None:
@@ -344,11 +433,14 @@ def update_env(
         The list of required packages for the model. If none is passed, the
         list of existing requirements is used and their versions are updated.
 
-    Returns
-    -------
-    None
     """
-    pass
+
+    with open(Path(path) / "config.json") as f:
+        config = json.load(f)
+
+    config["sklearn"]["environment"] = requirements
+
+    dump_json(Path(path) / "config.json", config)
 
 
 def push(
@@ -358,6 +450,7 @@ def push(
     token: str | None = None,
     commit_message: str | None = None,
     create_remote: bool = False,
+    private: bool | None = None,
 ) -> None:
     """Pushes the contents of a model repo to Hugging Face Hub.
 
@@ -379,12 +472,20 @@ def push(
     commit_message: str, optional
         The commit message to be used when pushing to the repo.
 
-    create_remote: bool, optional
+    create_remote: bool, default=False
         Whether to create the remote repository if it doesn't exist. If the
         remote repository doesn't exist and this parameter is ``False``, it
         raises an error. Otherwise it checks if the remote repository exists,
         and would create it if it doesn't.
 
+    private: bool, default=None
+        Whether the remote repository should be public or private. If ``True``
+        or ``False`` is passed, this method will set the private/public status
+        of the remote repository, regardless of it already existing or not. If
+        ``None``, no change is applied.
+
+        .. versionadded:: 0.3
+
     Returns
     -------
     None
@@ -399,10 +500,12 @@ def push(
     client = HfApi()
 
     if create_remote:
-        try:
-            client.model_info(repo_id=repo_id, token=token)
-        except HTTPError:
-            client.create_repo(repo_id=repo_id, token=token, repo_type="model")
+        client.create_repo(
+            repo_id=repo_id, token=token, repo_type="model", exist_ok=True
+        )
+
+    if private is not None:
+        client.update_repo_visibility(repo_id=repo_id, private=private, token=token)
 
     client.upload_folder(
         repo_id=repo_id,
@@ -506,7 +609,10 @@ def download(
     dst = Path(dst)
     if dst.exists() and bool(next(dst.iterdir(), None)):
         raise OSError("None-empty dst path already exists!")
-    dst.rmdir()
+
+    # remove the folder only if it's empty and it exists
+    if dst.exists():
+        dst.rmdir()
 
     cached_folder = snapshot_download(
         repo_id=repo_id, revision=revision, use_auth_token=token, **kwargs
@@ -547,8 +653,13 @@ def get_model_output(repo_id: str, data: Any, token: Optional[str] = None) -> An
     If there are warnings or exceptions during inference, this function raises
     a :class:`RuntimeError` including the original errors and warnings
     returned from the server.
+
+    Also note that if the model repo is private, the inference API would not be
+    available.
     """
-    model_info = HfApi().model_info(repo_id=repo_id, token=token)
+    # TODO: the "type: ignore" should eventually become unncessary when hf_hub
+    # is updated
+    model_info = HfApi().model_info(repo_id=repo_id, use_auth_token=token)  # type: ignore
     if not model_info.pipeline_tag:
         raise ValueError(
             f"Repo {repo_id} has no pipeline tag. You should set a valid 'task' in"
