@@ -5,25 +5,30 @@ import re
 import shutil
 import tempfile
 import warnings
+from importlib import metadata
 from pathlib import Path
 from uuid import uuid4
 
 import numpy as np
 import pandas as pd
 import pytest
+import sklearn
 from flaky import flaky
 from huggingface_hub import HfApi
+from huggingface_hub.utils import RepositoryNotFoundError
 from sklearn.datasets import load_diabetes, load_iris
 from sklearn.linear_model import LinearRegression, LogisticRegression
 
 from skops import card
 from skops.hub_utils import (
+    add_files,
     download,
     get_config,
     get_model_output,
     get_requirements,
     init,
     push,
+    update_env,
 )
 from skops.hub_utils._hf_hub import (
     _create_config,
@@ -32,11 +37,13 @@ from skops.hub_utils._hf_hub import (
     _validate_folder,
 )
 from skops.hub_utils.tests.common import HF_HUB_TOKEN
-from skops.utils.fixes import metadata, path_unlink
+from skops.io import dump
 
 iris = load_iris(as_frame=True, return_X_y=False)
 diabetes = load_diabetes(as_frame=True, return_X_y=False)
 
+IS_SKLEARN_DEV_BUILD = "dev" in sklearn.__version__
+
 
 @pytest.fixture
 def temp_path():
@@ -69,39 +76,67 @@ def get_regressor():
 
 
 @pytest.fixture(scope="session")
-def classifier_pickle(repo_path):
-    # Create a simple pickle file for the purpose of testing
+def classifier(repo_path, config_json):
+    # Create a simple model file for the purpose of testing
     clf = get_classifier()
-    path = repo_path / "model.pickle"
+    config_path, file_format = config_json
+    model_file = CONFIG[file_format]["sklearn"]["model"]["file"]
+    path = repo_path / model_file
 
     try:
-        with open(path, "wb") as f:
-            pickle.dump(clf, f)
+        if file_format == "pickle":
+            with open(path, "wb") as f:
+                pickle.dump(clf, f)
+        elif file_format == "skops":
+            dump(clf, path)
         yield path
     finally:
-        path_unlink(path, missing_ok=True)
+        path.unlink(missing_ok=True)
 
 
 CONFIG = {
-    "sklearn": {
-        "environment": ['scikit-learn="1.1.1"'],
-        "model": {"file": "model.pickle"},
-    }
+    "pickle": {
+        "sklearn": {
+            "environment": ['scikit-learn="1.1.1"'],
+            "model": {"file": "model.pickle"},
+        }
+    },
+    "skops": {
+        "sklearn": {
+            "environment": ['scikit-learn="1.1.1"'],
+            "model": {"file": "model.skops"},
+        }
+    },
 }
 
 
-@pytest.fixture(scope="session")
-def config_json(repo_path):
+@pytest.fixture(scope="session", params=["skops", "pickle"])
+def config_json(repo_path, request):
     path = repo_path / "config.json"
     try:
         with open(path, "w") as f:
-            json.dump(CONFIG, f)
-        yield path
+            json.dump(CONFIG[request.param], f)
+        yield path, request.param
     finally:
-        path_unlink(path, missing_ok=True)
+        path.unlink(missing_ok=True)
+
+
+def test_validate_format(classifier):
+    dir_path = tempfile.mkdtemp()
+    shutil.rmtree(dir_path)
+    with pytest.raises(ValueError, match="Cannot determine the input file*"):
+        init(
+            model=classifier,
+            requirements=["scikit-learn"],
+            dst=dir_path,
+            task="tabular-classification",
+            data=iris.data,
+            model_format="dummy",
+        )
 
 
 def test_validate_folder(config_json):
+    config_path, file_format = config_json
     _, file_path = tempfile.mkstemp()
     dir_path = tempfile.mkdtemp()
     with pytest.raises(TypeError, match="The given path is not a directory."):
@@ -118,11 +153,12 @@ def test_validate_folder(config_json):
     ):
         _validate_folder(path=dir_path)
 
-    shutil.copy2(config_json, dir_path)
-    with pytest.raises(TypeError, match="Model file model.pickle does not exist."):
+    shutil.copy2(config_path, dir_path)
+    model_file = CONFIG[file_format]["sklearn"]["model"]["file"]
+    with pytest.raises(TypeError, match=f"Model file {model_file} does not exist."):
         _validate_folder(path=dir_path)
 
-    (Path(dir_path) / "model.pickle").touch()
+    (Path(dir_path) / model_file).touch()
 
     # this should now work w/o an error
     _validate_folder(path=dir_path)
@@ -204,11 +240,11 @@ def test_create_config_invalid_text_data(temp_path):
         )
 
 
-def test_atomic_init(classifier_pickle, temp_path):
+def test_atomic_init(classifier, temp_path):
     with pytest.raises(ValueError):
         # this fails since we're passing an invalid task.
         init(
-            model=classifier_pickle,
+            model=classifier,
             requirements=["scikit-learn"],
             dst=temp_path,
             task="tabular-classification",
@@ -218,7 +254,7 @@ def test_atomic_init(classifier_pickle, temp_path):
     # this passes even though the above init has failed once, on the same
     # destination path.
     init(
-        model=classifier_pickle,
+        model=classifier,
         requirements=["scikit-learn"],
         dst=temp_path,
         task="tabular-classification",
@@ -226,12 +262,12 @@ def test_atomic_init(classifier_pickle, temp_path):
     )
 
 
-def test_init_invalid_task(classifier_pickle, temp_path):
+def test_init_invalid_task(classifier, temp_path):
     with pytest.raises(
         ValueError, match="Task invalid not supported. Supported tasks are"
     ):
         init(
-            model=classifier_pickle,
+            model=classifier,
             requirements=["scikit-learn"],
             dst=temp_path,
             task="invalid",
@@ -239,14 +275,15 @@ def test_init_invalid_task(classifier_pickle, temp_path):
         )
 
 
-def test_init(classifier_pickle, config_json):
+def test_init(classifier, config_json):
+    config_path, file_format = config_json
     # create a temp directory and delete it, we just need a unique name.
     dir_path = tempfile.mkdtemp()
     shutil.rmtree(dir_path)
 
     version = metadata.version("scikit-learn")
     init(
-        model=classifier_pickle,
+        model=classifier,
         requirements=[f'scikit-learn="{version}"'],
         dst=dir_path,
         task="tabular-classification",
@@ -257,7 +294,7 @@ def test_init(classifier_pickle, config_json):
     # it should fail a second time since the folder is no longer empty.
     with pytest.raises(OSError, match="None-empty dst path already exists!"):
         init(
-            model=classifier_pickle,
+            model=classifier,
             requirements=[f'scikit-learn="{version}"'],
             dst=dir_path,
             task="tabular-classification",
@@ -265,7 +302,8 @@ def test_init(classifier_pickle, config_json):
         )
 
 
-def test_init_no_warning_or_error(classifier_pickle, config_json):
+def test_init_no_warning_or_error(classifier, config_json):
+    config_path, file_format = config_json
     # for the happy path, there should be no warning
     dir_path = tempfile.mkdtemp()
     shutil.rmtree(dir_path)
@@ -274,7 +312,7 @@ def test_init_no_warning_or_error(classifier_pickle, config_json):
     with warnings.catch_warnings():
         warnings.simplefilter("error")
         init(
-            model=classifier_pickle,
+            model=classifier,
             requirements=[f'scikit-learn="{version}"'],
             dst=dir_path,
             task="tabular-classification",
@@ -283,6 +321,7 @@ def test_init_no_warning_or_error(classifier_pickle, config_json):
 
 
 def test_model_file_does_not_exist_raises(repo_path, config_json):
+    config_path, file_format = config_json
     # when the model file does not exist, raise an OSError
     model_path = repo_path / "foobar.pickle"
     dir_path = tempfile.mkdtemp()
@@ -298,20 +337,22 @@ def test_model_file_does_not_exist_raises(repo_path, config_json):
             task="tabular-classification",
             data=iris.data,
         )
-    path_unlink(model_path, missing_ok=True)
+    model_path.unlink(missing_ok=True)
 
 
-def test_init_empty_model_file_warns(repo_path, config_json):
+def test_init_empty_model_file_errors(repo_path, config_json):
+    config_path, file_format = config_json
     # when model file is empty, warn users
-    model_path = repo_path / "foobar.pickle"
-    with open(model_path, "wb"):
-        pass
+    model_path = Path(repo_path / "foobar.pickle")
+    model_path.touch()
 
     dir_path = tempfile.mkdtemp()
     shutil.rmtree(dir_path)
     version = metadata.version("scikit-learn")
 
-    with pytest.warns() as rec:
+    with pytest.raises(
+        RuntimeError, match=re.escape(f"Model file '{model_path}' is empty.")
+    ):
         init(
             model=model_path,
             requirements=[f'scikit-learn="{version}"'],
@@ -319,9 +360,7 @@ def test_init_empty_model_file_warns(repo_path, config_json):
             task="tabular-classification",
             data=iris.data,
         )
-        assert len(rec) == 1
-        assert rec[0].message.args[0] == f"Model file '{model_path}' is empty."
-    path_unlink(model_path, missing_ok=True)
+    model_path.unlink(missing_ok=True)
 
 
 @pytest.mark.network
@@ -331,14 +370,15 @@ def test_push_download(
     explicit_create,
     repo_path,
     destination_path,
-    classifier_pickle,
+    classifier,
     config_json,
 ):
+    config_path, file_format = config_json
     client = HfApi()
 
     version = metadata.version("scikit-learn")
     init(
-        model=classifier_pickle,
+        model=classifier,
         requirements=[f'scikit-learn="{version}"'],
         dst=destination_path,
         task="tabular-classification",
@@ -355,13 +395,20 @@ def test_push_download(
         token=HF_HUB_TOKEN,
         commit_message="test message",
         create_remote=True,
+        private=True,
     )
 
-    with pytest.raises(OSError, match="None-empty dst path already exists!"):
-        download(repo_id=repo_id, dst=destination_path)
+    with pytest.raises(
+        RepositoryNotFoundError,
+        match="If the repo is private, make sure you are authenticated.",
+    ):
+        download(repo_id=repo_id, dst="/tmp/test")
 
-    files = client.list_repo_files(repo_id=repo_id, token=HF_HUB_TOKEN)
-    for f_name in [classifier_pickle.name, config_json.name]:
+    with pytest.raises(OSError, match="None-empty dst path already exists!"):
+        download(repo_id=repo_id, dst=destination_path, token=HF_HUB_TOKEN)
+
+    files = client.list_repo_files(repo_id=repo_id, use_auth_token=HF_HUB_TOKEN)
+    for f_name in [classifier.name, config_path.name]:
         assert f_name in files
 
     try:
@@ -373,7 +420,19 @@ def test_push_download(
         client.delete_repo(repo_id=repo_id, token=HF_HUB_TOKEN)
 
 
+@pytest.fixture
+def repo_path_for_inference():
+    # Create a separate path for test_inference so that the test does not have
+    # any side-effect on existing tests
+    with tempfile.TemporaryDirectory(prefix="skops-test-sample-repo") as repo_path:
+        yield Path(repo_path)
+
+
 @pytest.mark.network
+@pytest.mark.inference
+@pytest.mark.skipif(
+    IS_SKLEARN_DEV_BUILD, reason="Inference tests cannot run with sklearn dev build"
+)
 @flaky(max_runs=3)
 @pytest.mark.parametrize(
     "model_func, data, task",
@@ -384,17 +443,26 @@ def test_push_download(
     ids=["classifier", "regressor"],
 )
 def test_inference(
+    config_json,
     model_func,
     data,
     task,
-    repo_path,
+    repo_path_for_inference,
     destination_path,
 ):
-    # test inference backend for classifier and regressor models.
-    client = HfApi()
+    # test inference backend for classifier and regressor models. This test can
+    # take a lot of time and be flaky.
+    config_path, file_format = config_json
+    if file_format != "pickle":
+        pytest.skip(
+            f"Inference only supports pickle at the moment. Given format: {file_format}"
+        )
 
+    client = HfApi()
+    repo_path = repo_path_for_inference
+    model_file = CONFIG[file_format]["sklearn"]["model"]["file"]
     model = model_func()
-    model_path = repo_path / "model.pickle"
+    model_path = repo_path / model_file
 
     with open(model_path, "wb") as f:
         pickle.dump(model, f)
@@ -408,7 +476,7 @@ def test_inference(
         data=data.data,
     )
 
-    # a model card is needed for inference engine to work.
+    # TODO: remove when card init at repo init is merged
     model_card = card.Card(
         model, metadata=card.metadata_from_config(Path(destination_path))
     )
@@ -423,6 +491,8 @@ def test_inference(
         token=HF_HUB_TOKEN,
         commit_message="test message",
         create_remote=True,
+        # api-inference doesn't support private repos for community projects.
+        private=False,
     )
 
     X_test = data.data.head(5)
@@ -431,17 +501,26 @@ def test_inference(
 
     # cleanup
     client.delete_repo(repo_id=repo_id, token=HF_HUB_TOKEN)
-    path_unlink(model_path, missing_ok=True)
+    model_path.unlink(missing_ok=True)
 
     assert np.allclose(output, y_pred)
 
 
-def test_get_config(repo_path):
+def test_get_config(repo_path, config_json):
+    config_path, file_format = config_json
     config = get_config(repo_path)
-    assert config == CONFIG
+
+    assert config == CONFIG[file_format]
     assert get_requirements(repo_path) == ['scikit-learn="1.1.1"']
 
 
+def test_update_env(repo_path, config_json):
+    # sanity check
+    assert get_requirements(repo_path) == ['scikit-learn="1.1.1"']
+    update_env(path=repo_path, requirements=['scikit-learn="1.1.2"'])
+    assert get_requirements(repo_path) == ['scikit-learn="1.1.2"']
+
+
 def test_get_example_input():
     """Test the _get_example_input function."""
     with pytest.raises(
@@ -489,3 +568,93 @@ def test_get_column_names_pandas_not_installed(pandas_not_installed):
     # is not installed and check that the function does not raise when pandas
     # import fails
     _get_column_names(np.ones((5, 10)))
+
+
+class TestAddFiles:
+    @pytest.fixture
+    def init_path(self, classifier, config_json):
+        # create temporary directory
+        dir_path = tempfile.mkdtemp()
+        shutil.rmtree(dir_path)
+
+        version = metadata.version("scikit-learn")
+        init(
+            model=classifier,
+            requirements=[f'scikit-learn="{version}"'],
+            dst=dir_path,
+            task="tabular-classification",
+            data=iris.data,
+        )
+        yield dir_path
+
+    @pytest.fixture
+    def some_file_0(self, temp_path):
+        filename = Path(temp_path) / "file0.txt"
+        with open(filename, "w") as f:
+            f.write("")
+        yield filename
+
+    @pytest.fixture
+    def some_file_1(self, temp_path):
+        filename = Path(temp_path) / "file1.txt"
+        with open(filename, "w") as f:
+            f.write("")
+        yield filename
+
+    def test_adding_one_file_path(self, init_path, some_file_0):
+        add_files(some_file_0, dst=init_path)
+        assert os.path.exists(Path(init_path) / some_file_0.name)
+
+    def test_adding_two_file_paths(self, init_path, some_file_0, some_file_1):
+        add_files(some_file_0, some_file_1, dst=init_path)
+        assert os.path.exists(Path(init_path) / some_file_0.name)
+        assert os.path.exists(Path(init_path) / some_file_1.name)
+
+    def test_adding_one_file_str(self, init_path, some_file_0):
+        add_files(str(some_file_0), dst=init_path)
+        assert os.path.exists(Path(init_path) / some_file_0.name)
+
+    def test_adding_two_files_str(self, init_path, some_file_0, some_file_1):
+        add_files(str(some_file_0), str(some_file_1), dst=init_path)
+        assert os.path.exists(Path(init_path) / some_file_0.name)
+        assert os.path.exists(Path(init_path) / some_file_1.name)
+
+    def test_adding_str_and_path(self, init_path, some_file_0, some_file_1):
+        add_files(str(some_file_0), some_file_1, dst=init_path)
+        assert os.path.exists(Path(init_path) / some_file_0.name)
+        assert os.path.exists(Path(init_path) / some_file_1.name)
+
+    def test_dst_does_not_exist_raises(self, some_file_0):
+        dst = tempfile.mkdtemp()
+        shutil.rmtree(dst)
+        msg = (
+            rf"Could not find \'{re.escape(dst)}\', did you run "
+            r"\'skops.hub_utils.init\' first\?"
+        )
+        with pytest.raises(FileNotFoundError, match=msg):
+            add_files(some_file_0, dst=dst)
+
+    def test_file_does_not_exist_raises(self, init_path, some_file_0):
+        non_existing_file = "foobar.baz"
+        msg = r"File \'foobar.baz\' could not be found."
+        with pytest.raises(FileNotFoundError, match=msg):
+            add_files(some_file_0, non_existing_file, dst=init_path)
+
+    def test_adding_existing_file_works_if_exist_ok(self, init_path, some_file_0):
+        add_files(some_file_0, dst=init_path)
+        assert os.path.exists(Path(init_path) / some_file_0.name)
+        add_files(some_file_0, dst=init_path, exist_ok=True)
+        assert os.path.exists(Path(init_path) / some_file_0.name)
+
+    def test_adding_existing_file_raises(self, init_path, some_file_0):
+        # first time around no warning
+        with warnings.catch_warnings():
+            warnings.simplefilter("error")
+            add_files(some_file_0, dst=init_path, exist_ok=False)
+
+        msg = (
+            f"File '{re.escape(some_file_0.name)}' already found "
+            f"at '{re.escape(init_path)}'."
+        )
+        with pytest.raises(FileExistsError, match=msg):
+            add_files(some_file_0, dst=init_path)
